==> rust-gitlab-v0.1102.0.zip: rust-gitlab-v0.1102.0/src/hooks.rs <==
// Copyright 2016 Kitware, Inc.
//
// Licensed under the Apache License, Version 2.0 <LICENSE-APACHE or
// http://www.apache.org/licenses/LICENSE-2.0> or the MIT license
// <LICENSE-MIT or http://opensource.org/licenses/MIT>, at your
// option. This file may not be copied, modified, or distributed
// except according to those terms.

//! Hook structures
//!
//! Hooks are received from Gitlab when registered as a system or web hook listener.
//!
//! Gitlab does not have consistent structures for its hooks, so they often change from
//! version to version.

use crates::serde::{Deserialize, Deserializer};
use crates::serde::de::{Error, Unexpected};
use crates::serde_json::{self, Value};

use systemhooks::SystemHook;
use webhooks::WebHook;

#[derive(Debug, Clone)]
/// A deserializable structure for all Gitlab hooks.
pub enum GitlabHook {
    /// A system hook.
    System(SystemHook),
    /// A web hook from a specific project.
    Web(WebHook),
}

impl<'de> Deserialize<'de> for GitlabHook {
    fn deserialize<D>(deserializer: D) -> Result<Self, D::Error>
        where D: Deserializer<'de>,
    {
        let val = <Value as Deserialize>::deserialize(deserializer)?;

        // Look for `object_kind` first because some web hooks also have `event_name` which would
        // cause a false match here.
        let hook_res = if val.pointer("/object_kind").is_some() {
            serde_json::from_value(val).map(GitlabHook::Web)
        } else if val.pointer("/event_name").is_some() {
            serde_json::from_value(val).map(GitlabHook::System)
        } else {
            return Err(D::Error::missing_field("either object_kind or event_name"));
        };

        hook_res.map_err(|err| {
            D::Error::invalid_value(Unexpected::Other("gitlab hook"),
                                    &format!("{:?}", err).as_str())
        })
    }
}

==> rust-gitlab-v0.1111.0.zip: rust-gitlab-v0.1111.0/src/hooks.rs <==
// Licensed under the Apache License, Version 2.0 <LICENSE-APACHE or
// http://www.apache.org/licenses/LICENSE-2.0> or the MIT license
// <LICENSE-MIT or http://opensource.org/licenses/MIT>, at your
// option. This file may not be copied, modified, or distributed
// except according to those terms.

//! Hook structures
//!
//! Hooks are received from Gitlab when registered as a system or web hook listener.
//!
//! Gitlab does not have consistent structures for its hooks, so they often change from
//! version to version.

use crates::serde::de::{Error, Unexpected};
use crates::serde::{Deserialize, Deserializer};
use crates::serde_json::{self, Value};

use systemhooks::SystemHook;
use webhooks::WebHook;

#[derive(Debug, Clone)]
/// A deserializable structure for all Gitlab hooks.
pub enum GitlabHook {
    /// A system hook.
    System(SystemHook),
    /// A web hook from a specific project.
    Web(WebHook),
}

impl<'de> Deserialize<'de> for GitlabHook {
    fn deserialize<D>(deserializer: D) -> Result<Self, D::Error>
    where
        D: Deserializer<'de>,
    {
        let val = <Value as Deserialize>::deserialize(deserializer)?;

        // Look for `object_kind` first because some web hooks also have `event_name` which would
        // cause a false match here.
        let hook_res = if val.pointer("/object_kind").is_some() {
            serde_json::from_value(val).map(GitlabHook::Web)
        } else if val.pointer("/event_name").is_some() {
            serde_json::from_value(val).map(GitlabHook::System)
        } else {
            return Err(D::Error::missing_field("either object_kind or event_name"));
        };

        hook_res.map_err(|err| {
            D::Error::invalid_value(
                Unexpected::Other("gitlab hook"),
                &format!("{:?}", err).as_str(),
            )
        })
    }
}
